==> jsonrpcclient-2.2.3.tar.gz: jsonrpcclient-2.2.3/jsonrpcclient/tornado_client.py <==
'''
TornadoClient
*************

Represents an endpoint to communicate with using Tornado asynchronous HTTP
client::

    from tornado import gen, ioloop

    @gen.coroutine
    def test():
        proxy = TornadoClient('http://example.com/api')
        result = yield proxy.some_method(42)
        print(result)

    ioloop.IOLoop.instance().run_sync(test)

'''

from tornado.httpclient import AsyncHTTPClient, HTTPError
from tornado.concurrent import Future

from .client import Client


class TornadoClient(Client):
    '''
    :param endpoint: The server address.
    :param async_http_client_class: Tornado asynchronous HTTP client class.
    :param kwargs: Keyword arguments to pass to the client initialiser.
    '''

    _default_headers = {
        'Content-Type' : 'application/json',
        'Accept'       : 'application/json'
    }


    def __init__(self, endpoint, async_http_client_class=AsyncHTTPClient, \
            **kwargs):
        super(TornadoClient, self).__init__(endpoint)

        self.http_client = async_http_client_class(**kwargs)

    def _process_response(self, response_future):
        future = Future()

        def process_response_callback(response_future):
            try:
                response = response_future.result()
                result = super(TornadoClient, self)._process_response(
                    response.body.decode())
            except Exception as ex: # pylint: disable=broad-except
                future.set_exception(ex)
            else:
                future.set_result(result)

        response_future.add_done_callback(process_response_callback)

        return future

    def _log_response_callback(self, future):
        ex = future.exception()
        if ex:
            if isinstance(ex, HTTPError):
                body = ex.response.body.decode() \
                    if ex.response and ex.response.body else None
                self._log_response(body, {
                    'http_code'    : ex.code,
                    'http_reason'  : ex.message,
                })
        else:
            response = future.result()
            self._log_response(response.body.decode(), {
                'http_code'    : response.code,
                'http_reason'  : response.reason,
                'http_headers' : response.headers
            })

    def _send_message(self, request, **kwargs):
        """Transport the message to the server and return the response.

        :param request: The JSON-RPC request string.
        :param kwargs: Keyword arguments to the Tornado request.
        :return: The response (a string for requests, None for notifications).
        """

        self._log_request(request)

        headers = dict(self._default_headers)
        headers.update(kwargs.get('headers', {}))

        future = self.http_client.fetch(self.endpoint, method='POST', \
            body=request, headers=headers, **kwargs)

        future.add_done_callback(self._log_response_callback)

        return future

==> jsonrpcclient-2.2.4.tar.gz: jsonrpcclient-2.2.4/jsonrpcclient/tornado_client.py <==
"""
TornadoClient
*************

Represents an endpoint to communicate with using Tornado asynchronous HTTP
client::

    from tornado import ioloop

    async def test():
        client = TornadoClient('http://example.com/api')
        result = await yield client.some_method(42)
        print(result)

    ioloop.IOLoop.instance().run_sync(test)
"""
from functools import partial

from tornado.httpclient import AsyncHTTPClient, HTTPError
from tornado.concurrent import Future

from .client import Client


class TornadoClient(Client):
    """
    :param endpoint: The server address.
    :param async_http_client_class: Tornado asynchronous HTTP client class.
    :param kwargs: Keyword arguments to pass to the client initialiser.
    """

    __DEFAULT_HEADERS = {
        'Content-Type' : 'application/json',
        'Accept'       : 'application/json'
    }


    def __init__(self, endpoint, async_http_client_class=AsyncHTTPClient, \
            **kwargs):
        super(TornadoClient, self).__init__(endpoint)
        self.http_client = async_http_client_class(**kwargs)

    def _request_sent(self, future, response):
        """Callback when request has been sent"""
        if response.error:
            future.set_exception(response.error)
        else:
            future.set_result(self._process_response(response.body.decode(), {
                'http_code': response.code, 'http_reason': response.reason,
                'http_headers' : response.headers}))

    def _send_message(self, request, **kwargs):
        """Transport the message to the server and return the response.

        :param request: The JSON-RPC request string.
        :param kwargs: Keyword arguments to the Tornado request.
        :return: The response (a string for requests, None for notifications).
        """
        headers = dict(self.__DEFAULT_HEADERS)
        headers.update(kwargs.get('headers', {}))

        future = Future()
        self.http_client.fetch(
            self.endpoint, method='POST', body=request, headers=headers,
            callback=partial(self._request_sent, future), **kwargs)
        return future
